==> simpledualwield-1.1.1%2B1.21.5-sources.jar: eva/dualwielding/util/DualWieldingOffhandAttack.java <==
package eva.dualwielding.util;

import eva.dualwielding.access.PlayerAccess;
import eva.dualwielding.init.ParticleInit;
import java.util.Optional;
import net.minecraft.class_1268;
import net.minecraft.class_1282;
import net.minecraft.class_1294;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1324;
import net.minecraft.class_1508;
import net.minecraft.class_1531;
import net.minecraft.class_1657;
import net.minecraft.class_1676;
import net.minecraft.class_1799;
import net.minecraft.class_1890;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_2743;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_3468;
import net.minecraft.class_3483;
import net.minecraft.class_3489;
import net.minecraft.class_3532;
import net.minecraft.class_5134;
import net.minecraft.class_9109;
import net.minecraft.class_9285;
import net.minecraft.class_9334;

public class DualWieldingOffhandAttack {

    public static float getOffhandAttackCooldownProgressPerTick(class_1657 playerEntity) {
        class_1324 newInstance = new class_1324(class_5134.field_23723, spd -> playerEntity.method_5996(class_5134.field_23723));
        if (playerEntity.method_6047().method_7909().method_57347().method_57832(class_9334.field_49636)) {
            class_9285 component = playerEntity.method_6047().method_7909().method_57347().method_58694(class_9334.field_49636);
            assert component != null;
            component.comp_2393().forEach((mod) -> {
                newInstance.method_6202(mod.comp_2396());
            });
        }
        if (playerEntity.method_6079().method_7909().method_57347().method_57832(class_9334.field_49636)) {
            class_9285 component = playerEntity.method_6079().method_7909().method_57347().method_58694(class_9334.field_49636);
            assert component != null;
            component.comp_2393().forEach((mod) -> {
                newInstance.method_61163(mod.comp_2396());
            });
        }
        return (float) (1.0 / newInstance.method_6194() * 20.0);
    }

    public static void offhandAttack(class_1657 playerEntity, class_1297 target) {
        if (!target.method_5732()) {
            return;
        }
        if (target.method_5698(playerEntity)) {
            return;
        }
        target.field_6008 = 0;

        float f = (float) playerEntity.method_45325(class_5134.field_23721);
        if (!playerEntity.method_37908().method_8608()) {
            class_1324 newInstance = new class_1324(class_5134.field_23721,
                    att -> playerEntity.method_5996(class_5134.field_23721));
            if (playerEntity.method_6047().method_7909().method_57347().method_57832(class_9334.field_49636)) {
                class_9285 component = playerEntity.method_6047().method_7909().method_57347().method_58694(class_9334.field_49636);
                //.filtered((ATTRIBUTE_MODIFIERS) -> true);
                assert component != null;
                component.comp_2393().forEach((mod) -> {
                    newInstance.method_6202(mod.comp_2396());
                });
            }
            if (playerEntity.method_6079().method_7909().method_57347().method_57832(class_9334.field_49636)) {
                class_9285 component = playerEntity.method_6079().method_7909().method_57347().method_58694(class_9334.field_49636);
                assert component != null;
                component.comp_2393().forEach((mod) -> {
                    newInstance.method_61163(mod.comp_2396());
                });
            }
            f = (float) newInstance.method_6194();
        }
        class_1799 itemStack = playerEntity.method_5998(class_1268.field_5810);
        class_1282 damageSource = Optional.ofNullable(itemStack.method_7909().method_64193(playerEntity)).orElse(playerEntity.method_48923().method_48802(playerEntity));
        float g = playerEntity.method_59903(target, f, damageSource) - f;
        float h = ((PlayerAccess) playerEntity).getAttackCooldownProgressDualOffhand(0.5F);
        f *= 0.2F + h * h * 0.8F;
        g *= h;
        ((PlayerAccess) playerEntity).resetLastDualOffhandAttackTicks();
        if (target.method_5864().method_20210(class_3483.field_51503)
                && target instanceof class_1676 projectileEntity
                && projectileEntity.method_59859(class_9109.field_51509, playerEntity, playerEntity, true)) {
            playerEntity.method_37908().method_54762(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14914, playerEntity.method_5634());
        } else {
            if (f > 0.0F || g > 0.0F) {
                boolean bl = h > 0.9F;
                boolean bl2;
                if (playerEntity.method_5624() && bl) {
                    playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14999, playerEntity.method_5634(), 1.0F, 1.0F);
                    bl2 = true;
                } else {
                    bl2 = false;
                }

                f += itemStack.method_7909().method_58403(target, f, damageSource);
                boolean bl3 = bl
                        && playerEntity.field_6017 > 0.0
                        && !playerEntity.method_24828()
                        && !playerEntity.method_6101()
                        && !playerEntity.method_5799()
                        && !playerEntity.method_6059(class_1294.field_5919)
                        && !playerEntity.method_5765()
                        && target instanceof class_1309
                        && !playerEntity.method_5624();
                if (bl3) {
                    f *= 1.5F;
                }

                float i = f + g;
                boolean bl4 = false;
                if (bl && !bl3 && !bl2 && playerEntity.method_24828()) {
                    double d = playerEntity.method_60478().method_37268();
                    double e = playerEntity.method_6029() * 2.5;
                    if (d < class_3532.method_33723(e) && playerEntity.method_5998(class_1268.field_5810).method_31573(class_3489.field_42611)) {
                        bl4 = true;
                    }
                }

                float j = 0.0F;
                if (target instanceof class_1309 livingEntity) {
                    j = livingEntity.method_6032();
                }

                class_243 vec3d = target.method_18798();
                boolean bl5 = target.method_64420(damageSource, i);
                if (bl5) {
                    float k = playerEntity.method_59924(target, damageSource) + (bl2 ? 1.0F : 0.0F);
                    if (k > 0.0F) {
                        if (target instanceof class_1309 livingEntity2) {
                            livingEntity2.method_6005(
                                    k * 0.5F, class_3532.method_15374(playerEntity.method_36454() * (float) (Math.PI / 180.0)), -class_3532.method_15362(playerEntity.method_36454() * (float) (Math.PI / 180.0))
                            );
                        } else {
                            target.method_5762(
                                    -class_3532.method_15374(playerEntity.method_36454() * (float) (Math.PI / 180.0)) * k * 0.5F, 0.1, class_3532.method_15362(playerEntity.method_36454() * (float) (Math.PI / 180.0)) * k * 0.5F
                            );
                        }

                        playerEntity.method_18799(playerEntity.method_18798().method_18805(0.6, 1.0, 0.6));
                        playerEntity.method_5728(false);
                    }

                    if (bl4) {
                        float l = 1.0F + (float)playerEntity.method_45325(class_5134.field_51577) * f;

                        for (class_1309 livingEntity3 : playerEntity.method_37908().method_18467(class_1309.class, target.method_5829().method_1009(1.0, 0.25, 1.0))) {
                            if (livingEntity3 != playerEntity
                                    && livingEntity3 != target
                                    && !playerEntity.method_5722(livingEntity3)
                                    && !(livingEntity3 instanceof class_1531 armorStandEntity && armorStandEntity.method_6912())
                                    && playerEntity.method_5858(livingEntity3) < 9.0) {
                                float m = playerEntity.method_59903(livingEntity3, l, damageSource) * h;
                                if (playerEntity.method_37908() instanceof class_3218 serverWorld && livingEntity3.method_64397(serverWorld, damageSource, m)) {
                                    livingEntity3.method_6005(
                                            0.4F, class_3532.method_15374(playerEntity.method_36454() * (float) (Math.PI / 180.0)), -class_3532.method_15362(playerEntity.method_36454() * (float) (Math.PI / 180.0))
                                    );
                                    assert damageSource != null;
                                    class_1890.method_60107(serverWorld, livingEntity3, damageSource);
                                }
                            }
                        }

                        playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14706, playerEntity.method_5634(), 1.0F, 1.0F);
                        double posOne = -class_3532.method_15374(playerEntity.method_36454() * ((float) Math.PI / 180));
                        double posTwo = class_3532.method_15362(playerEntity.method_36454() * ((float) Math.PI / 180));
                        ((class_3218) playerEntity.method_37908()).method_65096(ParticleInit.OFFHAND_SWEEPING, playerEntity.method_23317() + posOne, playerEntity.method_23323(0.5D), playerEntity.method_23321() + posTwo, 0,
                                posOne, 0.0D, posTwo, 0.0D);
                    }

                    if (target instanceof class_3222 && target.field_6037) {
                        ((class_3222)target).field_13987.method_14364(new class_2743(target));
                        target.field_6037 = false;
                        target.method_18799(vec3d);
                    }

                    if (bl3) {
                        playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_15016, playerEntity.method_5634(), 1.0F, 1.0F);
                        playerEntity.method_7277(target);
                    }

                    if (!bl3 && !bl4) {
                        if (bl) {
                            playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14840, playerEntity.method_5634(), 1.0F, 1.0F);
                        } else {
                            playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14625, playerEntity.method_5634(), 1.0F, 1.0F);
                        }
                    }

                    if (g > 0.0F) {
                        playerEntity.method_7304(target);
                    }

                    playerEntity.method_6114(target);
                    class_1297 entity = target;
                    if (target instanceof class_1508) {
                        entity = ((class_1508)target).field_7007;
                    }

                    boolean bl6 = false;
                    if (playerEntity.method_37908() instanceof class_3218 serverWorld2) {
                        if (entity instanceof class_1309 livingEntity3x) {
                            bl6 = itemStack.method_7979(livingEntity3x, playerEntity);
                        }

                        class_1890.method_60107(serverWorld2, target, damageSource);
                    }

                    if (!playerEntity.method_37908().field_9236 && !itemStack.method_7960() && entity instanceof class_1309) {
                        if (bl6) {
                            itemStack.method_59979((class_1309)entity, playerEntity);
                        }

                        if (itemStack.method_7960()) {
                            if (itemStack == playerEntity.method_6047()) {
                                playerEntity.method_6122(class_1268.field_5810, class_1799.field_8037);
                            } else {
                                playerEntity.method_6122(class_1268.field_5808, class_1799.field_8037);
                            }
                        }
                    }

                    if (target instanceof class_1309) {
                        float n = j - ((class_1309)target).method_6032();
                        playerEntity.method_7339(class_3468.field_15399, Math.round(n * 10.0F));
                        if (playerEntity.method_37908() instanceof class_3218 && n > 2.0F) {
                            int o = (int)(n * 0.5);
                            ((class_3218)playerEntity.method_37908())
                                    .method_65096(class_2398.field_11209, target.method_23317(), target.method_23323(0.5), target.method_23321(), o, 0.1, 0.0, 0.1, 0.2);
                        }
                    }

                    playerEntity.method_7322(0.1F);
                } else {
                    playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14914, playerEntity.method_5634(), 1.0F, 1.0F);
                }
            }
        }
    }

}

==> simpledualwield-1.2.0%2B1.21.5-sources.jar: eva/dualwielding/util/DualWieldingOffhandAttack.java <==
package eva.dualwielding.util;

import eva.dualwielding.DualWieldingMain;
import eva.dualwielding.access.PlayerAccess;
import eva.dualwielding.init.ParticleInit;
import java.util.Optional;
import net.minecraft.class_1268;
import net.minecraft.class_1282;
import net.minecraft.class_1294;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1324;
import net.minecraft.class_1508;
import net.minecraft.class_1531;
import net.minecraft.class_1657;
import net.minecraft.class_1676;
import net.minecraft.class_1799;
import net.minecraft.class_1890;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_2743;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_3468;
import net.minecraft.class_3483;
import net.minecraft.class_3489;
import net.minecraft.class_3532;
import net.minecraft.class_5134;
import net.minecraft.class_9109;
import net.minecraft.class_9274;
import net.minecraft.class_9285;
import net.minecraft.class_9334;

public class DualWieldingOffhandAttack {

    public static class_1324 offSpeed;
    public static class_1324 offDamage;

    public static void setOffAttributes(class_1657 playerEntity) {
        offSpeed = new class_1324(class_5134.field_23723, spd -> playerEntity.method_5996(class_5134.field_23723));
        if (playerEntity.method_6047().method_7909().method_57347().method_57832(class_9334.field_49636)) {
            class_9285 component = playerEntity.method_6047().method_7909().method_57347().method_58694(class_9334.field_49636);
            assert component != null;
            for(class_9285.class_9287 mod : component.comp_2393()) {
                if (offSpeed.method_6196(mod.comp_2396().comp_2447()) && mod.comp_2397() == class_9274.field_49217
                    && mod.comp_2395() == class_5134.field_23723)
                    offSpeed.method_6202(mod.comp_2396());
                DualWieldingMain.LOGGER.info("{} {} {} {}", mod.comp_2395().method_55840(), mod.comp_2396().comp_2449(),
                        mod.comp_2396().comp_2450(), mod.comp_2396().comp_2447());
            }
//            component.modifiers().forEach((mod) -> {
//                offSpeed.removeModifier(mod.modifier());
//            });
            component.comp_2393().forEach((mod) -> {
                if (!offSpeed.method_6196(mod.comp_2396().comp_2447()) && mod.comp_2397() == class_9274.field_49217
                        && mod.comp_2395() == class_5134.field_23723)
                    offSpeed.method_26837(mod.comp_2396());
            });
        }

        offDamage = new class_1324(class_5134.field_23721,
                att -> playerEntity.method_5996(class_5134.field_23721));
        if (playerEntity.method_6079().method_7909().method_57347().method_57832(class_9334.field_49636)) {
            class_9285 component = playerEntity.method_6047().method_7909().method_57347().method_58694(class_9334.field_49636);
            //.filtered((ATTRIBUTE_MODIFIERS) -> true);
            assert component != null;
            component.comp_2393().forEach((mod) -> {
                if (offDamage.method_6196(mod.comp_2396().comp_2447()) && mod.comp_2397() == class_9274.field_49217
                        && mod.comp_2395() == class_5134.field_23721)
                    offDamage.method_6202(mod.comp_2396());
            });
            component.comp_2393().forEach((mod) -> {
                if (!offDamage.method_6196(mod.comp_2396().comp_2447()) && mod.comp_2397() == class_9274.field_49217
                        && mod.comp_2395() == class_5134.field_23721)
                    offDamage.method_26837(mod.comp_2396());
            });
        }

    }


    public static float getOffhandAttackCooldownProgressPerTick(class_1657 playerEntity) {
        if (offSpeed == null) setOffAttributes(playerEntity);
        return (float)((double)1.0F / offSpeed.method_6194() * (double)20.0F);
    }

    public static void offhandAttack(class_1657 playerEntity, class_1297 target) {
        if (!target.method_5732()) {
            return;
        }
        if (target.method_5698(playerEntity)) {
            return;
        }
        target.field_6008 = 0;

        float f = (float) offDamage.method_6194();

        class_1799 itemStack = playerEntity.method_5998(class_1268.field_5810);
        class_1282 damageSource = Optional.ofNullable(itemStack.method_7909().method_64193(playerEntity)).orElse(playerEntity.method_48923().method_48802(playerEntity));
        float g = playerEntity.method_59903(target, f, damageSource) - f;
        float h = ((PlayerAccess) playerEntity).getOffhandAttackCooldownProgress(0.5F);
        f *= 0.2F + h * h * 0.8F;
        g *= h;
        ((PlayerAccess) playerEntity).resetLastOffhandAttackTicks();
        if (target.method_5864().method_20210(class_3483.field_51503)
                && target instanceof class_1676 projectileEntity
                && projectileEntity.method_59859(class_9109.field_51509, playerEntity, playerEntity, true)) {
            playerEntity.method_37908().method_54762(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14914, playerEntity.method_5634());
        } else {
            if (f > 0.0F || g > 0.0F) {
                boolean bl = h > 0.9F;
                boolean bl2;
                if (playerEntity.method_5624() && bl) {
                    playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14999, playerEntity.method_5634(), 1.0F, 1.0F);
                    bl2 = true;
                } else {
                    bl2 = false;
                }

                f += itemStack.method_7909().method_58403(target, f, damageSource);
                boolean bl3 = bl
                        && playerEntity.field_6017 > 0.0
                        && !playerEntity.method_24828()
                        && !playerEntity.method_6101()
                        && !playerEntity.method_5799()
                        && !playerEntity.method_6059(class_1294.field_5919)
                        && !playerEntity.method_5765()
                        && target instanceof class_1309
                        && !playerEntity.method_5624();
                if (bl3) {
                    f *= 1.5F;
                }

                float i = f + g;
                boolean bl4 = false;
                if (bl && !bl3 && !bl2 && playerEntity.method_24828()) {
                    double d = playerEntity.method_60478().method_37268();
                    double e = playerEntity.method_6029() * 2.5;
                    if (d < class_3532.method_33723(e) && playerEntity.method_5998(class_1268.field_5810).method_31573(class_3489.field_42611)) {
                        bl4 = true;
                    }
                }

                float j = 0.0F;
                if (target instanceof class_1309 livingEntity) {
                    j = livingEntity.method_6032();
                }

                class_243 vec3d = target.method_18798();
                boolean bl5 = target.method_64420(damageSource, i);
                if (bl5) {
                    float k = playerEntity.method_59924(target, damageSource) + (bl2 ? 1.0F : 0.0F);
                    if (k > 0.0F) {
                        if (target instanceof class_1309 livingEntity2) {
                            livingEntity2.method_6005(
                                    k * 0.5F, class_3532.method_15374(playerEntity.method_36454() * (float) (Math.PI / 180.0)), -class_3532.method_15362(playerEntity.method_36454() * (float) (Math.PI / 180.0))
                            );
                        } else {
                            target.method_5762(
                                    -class_3532.method_15374(playerEntity.method_36454() * (float) (Math.PI / 180.0)) * k * 0.5F, 0.1, class_3532.method_15362(playerEntity.method_36454() * (float) (Math.PI / 180.0)) * k * 0.5F
                            );
                        }

                        playerEntity.method_18799(playerEntity.method_18798().method_18805(0.6, 1.0, 0.6));
                        playerEntity.method_5728(false);
                    }

                    if (bl4) {
                        float l = 1.0F + (float)playerEntity.method_45325(class_5134.field_51577) * f;

                        for (class_1309 livingEntity3 : playerEntity.method_37908().method_18467(class_1309.class, target.method_5829().method_1009(1.0, 0.25, 1.0))) {
                            if (livingEntity3 != playerEntity
                                    && livingEntity3 != target
                                    && !playerEntity.method_5722(livingEntity3)
                                    && !(livingEntity3 instanceof class_1531 armorStandEntity && armorStandEntity.method_6912())
                                    && playerEntity.method_5858(livingEntity3) < 9.0) {
                                float m = playerEntity.method_59903(livingEntity3, l, damageSource) * h;
                                if (playerEntity.method_37908() instanceof class_3218 serverWorld && livingEntity3.method_64397(serverWorld, damageSource, m)) {
                                    livingEntity3.method_6005(
                                            0.4F, class_3532.method_15374(playerEntity.method_36454() * (float) (Math.PI / 180.0)), -class_3532.method_15362(playerEntity.method_36454() * (float) (Math.PI / 180.0))
                                    );
                                    assert damageSource != null;
                                    class_1890.method_60107(serverWorld, livingEntity3, damageSource);
                                }
                            }
                        }

                        playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14706, playerEntity.method_5634(), 1.0F, 1.0F);
                        double posOne = -class_3532.method_15374(playerEntity.method_36454() * ((float) Math.PI / 180));
                        double posTwo = class_3532.method_15362(playerEntity.method_36454() * ((float) Math.PI / 180));
                        ((class_3218) playerEntity.method_37908()).method_65096(ParticleInit.OFFHAND_SWEEPING, playerEntity.method_23317() + posOne, playerEntity.method_23323(0.5D), playerEntity.method_23321() + posTwo, 0,
                                posOne, 0.0D, posTwo, 0.0D);
                    }

                    if (target instanceof class_3222 && target.field_6037) {
                        ((class_3222)target).field_13987.method_14364(new class_2743(target));
                        target.field_6037 = false;
                        target.method_18799(vec3d);
                    }

                    if (bl3) {
                        playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_15016, playerEntity.method_5634(), 1.0F, 1.0F);
                        playerEntity.method_7277(target);
                    }

                    if (!bl3 && !bl4) {
                        if (bl) {
                            playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14840, playerEntity.method_5634(), 1.0F, 1.0F);
                        } else {
                            playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14625, playerEntity.method_5634(), 1.0F, 1.0F);
                        }
                    }

                    if (g > 0.0F) {
                        playerEntity.method_7304(target);
                    }

                    playerEntity.method_6114(target);
                    class_1297 entity = target;
                    if (target instanceof class_1508) {
                        entity = ((class_1508)target).field_7007;
                    }

                    boolean bl6 = false;
                    if (playerEntity.method_37908() instanceof class_3218 serverWorld2) {
                        if (entity instanceof class_1309 livingEntity3x) {
                            bl6 = itemStack.method_7979(livingEntity3x, playerEntity);
                        }

                        class_1890.method_60107(serverWorld2, target, damageSource);
                    }

                    if (!playerEntity.method_37908().field_9236 && !itemStack.method_7960() && entity instanceof class_1309) {
                        if (bl6) {
                            itemStack.method_59979((class_1309)entity, playerEntity);
                        }

                        if (itemStack.method_7960()) {
                            if (itemStack == playerEntity.method_6047()) {
                                playerEntity.method_6122(class_1268.field_5810, class_1799.field_8037);
                            } else {
                                playerEntity.method_6122(class_1268.field_5808, class_1799.field_8037);
                            }
                        }
                    }

                    if (target instanceof class_1309) {
                        float n = j - ((class_1309)target).method_6032();
                        playerEntity.method_7339(class_3468.field_15399, Math.round(n * 10.0F));
                        if (playerEntity.method_37908() instanceof class_3218 && n > 2.0F) {
                            int o = (int)(n * 0.5);
                            ((class_3218)playerEntity.method_37908())
                                    .method_65096(class_2398.field_11209, target.method_23317(), target.method_23323(0.5), target.method_23321(), o, 0.1, 0.0, 0.1, 0.2);
                        }
                    }

                    playerEntity.method_7322(0.1F);
                } else {
                    playerEntity.method_37908().method_43128(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(), class_3417.field_14914, playerEntity.method_5634(), 1.0F, 1.0F);
                }
            }
        }
    }

}
